@@ -607,8 +607,9 @@ class CxxParser:
                         lex.return_token(ptok)
                         param = self._parse_template_type_parameter(tok, None)
                     else:
+                        lex.return_token(ptok)
                         param, _ = self._parse_parameter(
-                            ptok,
+                            tok,
                             TemplateNonTypeParam,
                             concept_ok=False,
                             deduce_this_ok=False,
@@ -802,13 +803,6 @@ class CxxParser:
             ),
         )
 
-    # fmt: off
-    _expr_operators = {
-        "<", ">", "|", "%", "^", "!", "*", "-", "+", "&", "=",
-        "&&", "||", "<<"
-    }
-    # fmt: on
-
     def _parse_requires(
         self,
         tok: LexToken,
@@ -817,38 +811,48 @@ class CxxParser:
 
         rawtoks: typing.List[LexToken] = []
 
-        # The easier case -- requires requires
-        if tok.type == "requires":
-            rawtoks.append(tok)
-            for tt in ("(", "{"):
-                tok = self._next_token_must_be(tt)
+        # The expression in a requires clause must be one of the following:
+        #  1) A primary expression
+        #  2) A sequence of (1) joined with &&
+        #  3) A sequence of (2) joined with ||
+        #
+        # In terms of validity, this is equivalent to a sequence of primary expressions
+        # joined with && and/or ||.
+        #
+        # In general, a primary expression is one of the following:
+        #  1) this
+        #  2) a literal
+        #  3) an identifier expression
+        #  4) a lambda expression
+        #  5) a fold expression
+        #  6) a requires expression
+        #  7) any parenthesized expression
+        #
+        # For simplicity, we only consider the following primary expressions:
+        #  1) parenthesized expressions (which includes fold expressions)
+        #  2) requires expressions
+        #  3) identifer expressions (possibly qualified, possibly templated)
+        while True:
+            if tok.type == "(":
                 rawtoks.extend(self._consume_balanced_tokens(tok))
-            # .. and that's it?
-
-        # this is either a parenthesized expression or a primary clause
-        elif tok.type == "(":
-            rawtoks.extend(self._consume_balanced_tokens(tok))
-        else:
-            while True:
-                if tok.type == "(":
-                    rawtoks.extend(self._consume_balanced_tokens(tok))
-                else:
-                    tok = self._parse_requires_segment(tok, rawtoks)
-
-                # If this is not an operator of some kind, we don't know how
-                # to proceed so let the next parser figure it out
-                if tok.value not in self._expr_operators:
-                    break
-
-                rawtoks.append(tok)
-
-                # check once more for compound operator?
                 tok = self.lex.token()
-                if tok.value in self._expr_operators:
-                    rawtoks.append(tok)
-                    tok = self.lex.token()
+            elif tok.type == "requires":
+                rawtoks.append(tok)
+                for tt in ("(", "{"):
+                    tok = self._next_token_must_be(tt)
+                    rawtoks.extend(self._consume_balanced_tokens(tok))
+                tok = self.lex.token()
+            else:
+                tok = self._parse_requires_segment(tok, rawtoks)
 
-            self.lex.return_token(tok)
+            if tok.value not in ("&&", "||"):
+                break
+
+            rawtoks.append(tok)
+
+            tok = self.lex.token()
+
+        self.lex.return_token(tok)
 
         return self._create_value(rawtoks)
 
@@ -1406,10 +1410,14 @@ class CxxParser:
     # Variable parsing
     #
 
-    def _parse_bitfield(self) -> int:
-        # is a integral constant expression... for now, just do integers
-        tok = self._next_token_must_be("INT_CONST_DEC")
-        return int(tok.value)
+    def _parse_bitfield(self) -> typing.Union[int, Value]:
+        # Convert to integer for backwards compat
+        tok = self.lex.token_if("INT_CONST_DEC")
+        if tok is not None:
+            return int(tok.value)
+
+        # But also support weird things too
+        return self._create_value(self._consume_value_until([], ";"))
 
     def _parse_field(
         self,
@@ -2244,7 +2252,9 @@ class CxxParser:
         # nonptr_fn is for parsing function types directly in template specialization
 
         while True:
-            tok = self.lex.token_if("*", "const", "volatile", "(")
+            tok = self.lex.token_if(
+                "*", "const", "volatile", "__restrict__", "restrict", "("
+            )
             if not tok:
                 break
 
@@ -2260,6 +2270,10 @@ class CxxParser:
                 if not isinstance(dtype, (Pointer, Type)):
                     raise self._parse_error(tok)
                 dtype.volatile = True
+            elif tok.type in ("__restrict__", "restrict"):
+                if not isinstance(dtype, (Pointer, Reference)):
+                    raise self._parse_error(tok)
+                dtype.restrict = True
             elif nonptr_fn:
                 # remove any inner grouping parens
                 while True:
@@ -2331,7 +2345,7 @@ class CxxParser:
 
             # peek at the next token and see if it's a paren. If so, it might
             # be a nasty function pointer
-            if self.lex.token_peek_if("("):
+            if self.lex.token_peek_if("(", "__restrict__", "restrict"):
                 dtype = self._parse_cv_ptr_or_fn(dtype, nonptr_fn)
 
         return dtype
